// Sample game script that implements some rudimentary HUD elements for Trespasser CE
// This is not meant as a HUD for players to use. It's only a script sample.

// 32-bit ARGB modulation color (in hex) for HW hud, alpha = 127, RGB = 255 255 255
const u4HudHWColorAlpha = 0x7fffffff;

// ---------------------------------------------------------------------------------------------------------------
// helper class that encapsulates HUD drawing for HW rendering (OnPaintHW3D)
class HardwareHUD
{
	bHW = true;

	texHealth = null;
	texAmmo = null;
	texDigits = null;

	constructor()
	{
		// init required resources (for simplicity of handling the script samples the images are located in the scripts dir)
		texHealth = TP_Engine.GetTexture("scripts/healthbox_a.tga");
		texAmmo = TP_Engine.GetTexture("scripts/ammobox_a.tga"); // an image with the evenly spaced digits "0 1 2 3 4 5 6 7 8 9", 16 pixels per char
		texDigits = TP_Engine.GetTexture("scripts/numbers_a.tga");
	}

	// draw a number in the range 0 to 999
	function DrawNumber(iNumber, xpos, ypos, bRightAlign = false)
	{
		// width and height for one digit
		local dw = 16;
		local dh = texDigits.GetHeight();

		if (iNumber > 999)
			iNumber = 999;
		else if (iNumber < 0)
			iNumber = 0;

		local s = iNumber.tostring();
		if (bRightAlign)
			xpos -= dw * s.len();

		foreach (c in s)
		{
			local i = c - '0';
			TP_Engine.DrawTexture(texDigits,  i * dw, 0, dw, dh,  xpos, ypos, dw, dh,  u4HudHWColorAlpha, 1);
			xpos += dw;
		}
	}

	function Draw()
	{
		if (!texDigits)
			return;

		local plr = TP_Engine.GetPlayer();
		if (!plr)
			return;

		local sw = TP_Engine.GetScreenWidth();
		local sh = TP_Engine.GetScreenHeight();

		const xmargin = 30;
		const ymargin = 30;

		// --------------
		// health display
		// --------------
		if (texHealth)
		{
			// dimensions for health HUD graphic
			local w = texHealth.GetWidth();
			local h = texHealth.GetHeight();

			// top-left corner for health HUD graphic
			local x = xmargin;
			local y = sh-h-ymargin;

			// top-left corner for health percentage text
			local tx = w;
			local ty = (h - texDigits.GetHeight()) / 2;

			// draw health box graphic
			TP_Engine.DrawTexture(texHealth,  0, 0, w, h,  x, y, w, h,  u4HudHWColorAlpha, 1);

			// draw health percentage (left-aligned)
			DrawNumber(plr.GetHealth().tointeger(), x+tx, y+ty);
		}

		// --------------
		// ammo display
		// --------------
		if (texAmmo)
		{
			// only draw this if player is holding a gun (with a valid max ammo count)
			local obj = plr.pinsHeldObject();
			if (obj instanceof CGun && obj.iMaxAmmo > 0)
			{
				local w = texAmmo.GetWidth();
				local h = texAmmo.GetHeight();

				// top-left corner for ammo HUD graphic
				local x = sw-w-xmargin;
				local y = sh-h-ymargin;

				// top-right corner for ammo count text
				const tx = 0;
				local ty = (h - texDigits.GetHeight()) / 2;

				// draw ammo box graphic
				TP_Engine.DrawTexture(texAmmo,  0, 0, w, h,  x, y, w, h,  u4HudHWColorAlpha, 1);
				
				// draw ammo count (right-aligned)
				DrawNumber(obj.iAmmo, x+tx, y+ty, true);
			}
		}
	}
}

// ---------------------------------------------------------------------------------------------------------------
// helper class that encapsulates HUD drawing for any rendering mode but serves as a fallback HUD for SW (OnPaint)
class SoftwareHUD
{
	bHW = false;

	bmHealth = null;
	bmAmmo = null;
	bmDigits = null;

	constructor()
	{
		// init required resources
		bmHealth = TP_Engine.GetBitmap("scripts/healthbox.bmp");
		bmAmmo = TP_Engine.GetBitmap("scripts/ammobox.bmp");
		bmDigits = TP_Engine.GetBitmap("scripts/numbers.bmp"); // width per digit is 16 pixels
	}

	// draw a number in the range 0 to 999
	function DrawNumber(iNumber, xpos, ypos, bRightAlign = false)
	{
		// width and height for one digit
		local dw = 16;
		local dh = bmDigits.GetHeight();

		if (iNumber > 999)
			iNumber = 999;
		else if (iNumber < 0)
			iNumber = 0;

		local s = iNumber.tostring();
		if (bRightAlign)
			xpos -= dw * s.len();

		foreach (c in s)
		{
			local i = c - '0';
			TP_Engine.DrawBitmap(bmDigits,  i * dw, 0, dw, dh,  xpos, ypos);
			xpos += dw;
		}
	}

	function Draw()
	{
		if (!bmDigits)
			return;

		local plr = TP_Engine.GetPlayer();
		if (!plr)
			return;

		local sw = TP_Engine.GetScreenWidth();
		local sh = TP_Engine.GetScreenHeight();

		const xmargin = 30;
		const ymargin = 30;

		// --------------
		// health display
		// --------------
		if (bmHealth)
		{
			// dimensions for health HUD graphic
			local w = bmHealth.GetWidth();
			local h = bmHealth.GetHeight();

			// top-left corner for health HUD graphic
			local x = xmargin;
			local y = sh-h-ymargin;

			// top-left corner for health percentage text
			local tx = w;
			local ty = (h - bmDigits.GetHeight()) / 2;

			// draw health box graphic
			TP_Engine.DrawBitmap(bmHealth,  0, 0, w, h,  x, y);

			// draw health percentage (left-aligned)
			DrawNumber(plr.GetHealth().tointeger(), x+tx, y+ty);
		}

		// --------------
		// ammo display
		// --------------
		if (bmAmmo)
		{
			// only draw this if player is holding a gun (with a valid max ammo count)
			local obj = plr.pinsHeldObject();
			if (obj instanceof CGun && obj.iMaxAmmo > 0)
			{
				local w = bmAmmo.GetWidth();
				local h = bmAmmo.GetHeight();

				// top-left corner for ammo HUD graphic
				local x = sw-w-xmargin;
				local y = sh-h-ymargin;

				// top-right corner for ammo count text
				const tx = 0;
				local ty = (h - bmDigits.GetHeight()) / 2;

				// draw ammo box graphic
				TP_Engine.DrawBitmap(bmAmmo,  0, 0, w, h,  x, y);

				// draw ammo count (right-aligned)
				DrawNumber(obj.iAmmo, x+tx, y+ty, true);
			}
		}
	}
}


// ---------------------------------------------------------------------------------------------------------------
// A game script that provides HUD functionality
class SampleHUD extends GameScript
{
	hud = null;

	function OnScriptInit(bSaveGame)
	{
		if ( TP_Engine.IsSoftwareMode() )
			hud = SoftwareHUD();
		else
			hud = HardwareHUD();
	}

	function OnPaintHW3D()
	{
		if (hud && hud.bHW)
			hud.Draw();
	}

	function OnPaint()
	{
		if (hud && !hud.bHW)
			hud.Draw();
	}

	// make the general purpose key 1 toggle a (hypothetical) map display
	function OnHandleInput()
	{
		// uCMD_USER1 - Toggle map
		if ( bKeyHit(uCMD_USER1) )
		{
			ToggleMap();
		}
	}

	// could defined some custom console commands
	/*function OnConsoleCommand(strCmd, strArgs)
	{
	print("Console command: " + strCmd);
		local cmd = strCmd.toupper();

		if (cmd == "HUD") 
		{
			// toggle HUD visibility
			hud.bVisible = !hud.bVisible;
			return true;
		}
		else if (cmd == "MAP") 
		{
			// toggle MAP visibility
			ToggleMap();
			return true;
		}

		return false;
	}*/

	// could setup an ATX102 action type to send a switch map message to us
	/*function OnMessage(strMessage, param)
	{
		if (strMessage == "ATXSwitchMap")
		{
			ChangeMap(param[0]);
		}
	}*/

	function ToggleMap()
	{
		// TODO: toggle map display on/off
	}
}
