// Sample scripts for a spitting Dino similar to ATX

// global array of all available SpitterProjectile projectile instances in the level
// (this array must be cleaned up when the level is destroyed)
gSpitProjectiles <- [];

// index of the current projectile to recycle in gSpitProjectiles
// (this index must be saved in savegames to correctly continue where it left off)
gSpitProjectileIndex <- 0;


// level script used to manage spitter-related global vars
class SpitterHelper extends LevelScript
{
	// declare nvram for savegame data
	nvram = null;

	constructor()
	{
		// create a squirrel table
		nvram = {};

		// create a table slot for our persistent variable
		nvram.iProjectileIndex <- 0;
	}

	function OnScriptInit(bSaveGame)
	{
		if (bSaveGame)
			// restore gSpitProjectileIndex from savegame
			gSpitProjectileIndex = nvram.iProjectileIndex;
		else
			// fresh level start, reset counter
			gSpitProjectileIndex = 0;
	}

	function OnPrepareSave()
	{
		// save gSpitProjectileIndex in savegame so it will continue cycle projectiles where it left off
		nvram.iProjectileIndex = gSpitProjectileIndex;
	}

	function OnWorldPurge()
	{
		// world database is going to be destroyed, make sure projectile list is cleared so it doesn't reference destroyed instances
		gSpitProjectiles = [];
	}
}


// assign this script to the projectile objects (assuming they're CInstance objects, otherwise the base class has to be changed here)
// spit projectile objects will automatically add themselves to the pool of avaialable projectiles
class SpitterProjectile extends CInstance
{
	constructor()
	{
		//print("SpitterProjectile ctor: " + GetInstanceName());

		// add this instance to the global pool of available projectiles
		gSpitProjectiles.append(this);
	}

	function Launch(v3Origin, r3OriginRot, v3Velocity, iVelImpulseDuration)
	{
		Teleport(v3Origin, r3OriginRot);

		MoveWithFixedVelocity(v3Velocity, iVelImpulseDuration, false);
	}
}


// a sitting dino
class Spitter extends CAnimal
{
	// nvram is a per-instance table that holds persistent state vars that are saved/restored with savegames
	nvram = null;

	// ===== Spitter configuration ======

	// minimum time between spitting (seconds)
	fMinLaunchInterval = 4.0;

	// distance from head (along the head's viewing direction) where the projectile spawns
	fProjectileSpawnDistFromHead = 0.45;

	// projecile speed
	fProjectileSpeed = 7.0;

	// velocity impulse length (in milliseconds)
	iProjectileImpulseLen = 60;

	// spit probability (1.0 = 100%)
	fLaunchProbaility = 0.5;

	// min time to wait after a failed probability check before trying again (seconds)
	fProbabilityInterval = 1.0;

	// squared min horizontal distance between head and target to allow spitting (must be >0)
	fMinHorizDistSq = 2.25 * 2.25;

	// squared min distance between head and target to allow spitting (must be >0)
	fMinDistSq = 2.5 * 2.5;

	// cosine of the cone half-angle inside with the target must be located
	// (the cone is like a viewing cone along the viewing direction of the dino head, with the tip in head)
	fMaxAngleCosine = cos(60.0 * 0.01745329);// the first value is the angle in degrees, the second is just a radian conversion factor

	// enable to make dino launch at target direction rather than in its viewing direction
	bAutoAim = true;

	// ==================================

	constructor()
	{
		//print("Spitter ctor: " + GetInstanceName());

		nvram = {};

		nvram.fNextLaunchTime <- -100.0;
		nvram.fNextLaunchProbabilityCheckTime <- -100.0;

		// optionally support TScript properties to override the default spitter configuration
		// TODO
	}

	function OnScriptInit(bSaveGame)
	{
		if (!bSaveGame)
		{
			nvram.fNextLaunchTime = -100.0;
			nvram.fNextLaunchProbabilityCheckTime = -100.0;
		}
	}

	function OnActBite(pinsTarget, v3TargetLoc, fTargetDist, bFeint)
	{
		if ( !gSpitProjectiles.len() )
		{
			print("Spitter error: no spit projectiles were found in the level, can't spit");
			return;
		}

		// why not let it spit on Feint as well?
		/*if (bFeint)
			return;*/

		// ==== timing and probability based launch decisions ====

		local fCurTime = TP_Engine.GetElapsedSimTime();

		if (fCurTime < nvram.fNextLaunchTime || fCurTime < nvram.fNextLaunchProbabilityCheckTime)
			return;

		if (TP_Engine.GetRandFloatRanged(0.0, 0.9999999) >= fLaunchProbaility)
		{
			nvram.fNextLaunchProbabilityCheckTime = fCurTime + fProbabilityInterval;
			return;
		}

		// ==== target location/distance based launch decisions ====

		local bboxHead = pbbBoundaryBox(EBoundaryBoxes.ebbHEAD);

		//local v3HeadPos = bboxHead.v3Pos();
		local v3HeadPos = v3GetHeadPos();

		local v3TargetDelta = v3TargetLoc - v3HeadPos;

		// make sure XY distance to target is large enough
		local fHorzDistSq = Vector3(v3TargetDelta.x, v3TargetDelta.y, 0).LengthSqr();
		if (fHorzDistSq < fMinHorizDistSq)
			return;

		// make sure 3D distance to target is large enough
		local fDistSq = v3TargetDelta.LengthSqr();
		if (fDistSq < fMinDistSq)
			return;

		// make sure target angle is within the allowed range

		local r3HeadRot = bboxHead.r3Rot();
		local v3HeadDir = Vector3.YAxis() * r3HeadRot;

		local v3HeadDirXY = Vector3(v3HeadDir.x, v3HeadDir.y, 0);
		local v3TargetDirXY = Vector3(v3TargetDelta.x, v3TargetDelta.y, 0);
		v3TargetDirXY.Normalise();

		local fAngleCosine = v3HeadDirXY.Dot(v3TargetDirXY);

		if (fAngleCosine < fMaxAngleCosine)
			return;

		// ==== huston, we are go for launch ====

		nvram.fNextLaunchProbabilityCheckTime = nvram.fNextLaunchTime = fCurTime + fMinLaunchInterval;

		local projectile = GetProjectile();

		local v3ProjSpawnPos = v3HeadPos + v3HeadDir * fProjectileSpawnDistFromHead;

		local v3ProjVel;
		if (bAutoAim)
			// technically this isn't entirely correct, to be fully accurate we would have to calc the dir between v3ProjSpawnPos and v3TargetLoc
			v3ProjVel = v3TargetDelta * (fProjectileSpeed / v3TargetDelta.Length());
		else
			v3ProjVel = v3HeadDir * fProjectileSpeed;

		projectile.Launch(v3ProjSpawnPos, r3HeadRot, v3ProjVel, iProjectileImpulseLen);
	}

	// returns a projectile from the projectile pool
	function GetProjectile()
	{
		gSpitProjectileIndex = (gSpitProjectileIndex + 1) % gSpitProjectiles.len();
		return gSpitProjectiles[gSpitProjectileIndex];
	}
}
